import logging
import math
import os
import re
import struct
from urllib.parse import quote, unquote

import requests
from fake_useragent import UserAgent


class SoGou:

    def __init__(self):
        self.ua = UserAgent()

    def download_all_lexicon(self, start=0, end=629):
        for page in range(start, end):
            url = f'https://pinyin.sogou.com/dict/cate/index/{page}'
            data = self.one_classify_lexicon(url)
            yield data

    def one_classify_lexicon(self, url):
        response = requests.get(url=url, headers={'User-Agent': self.ua.random}).text
        max_page = re.search('分类下共有(.+)个词库', response).group(1)
        urls = re.findall('http://download.pinyin.sogou.com/dict/download_cell.php?.+"', response)
        for page in range(2, math.ceil(int(max_page) / 10) + 1):
            response = requests.get(url=url + f'/default/{page}', headers={'User-Agent': self.ua.random}).text
            urls.extend(re.findall('http://download.pinyin.sogou.com/dict/download_cell.php?.+"', response))
        return map(lambda x: unquote(x[:-1]), urls)

    def search_name_lexicon(self, search_name):
        url_word = quote(search_name.encode('GBK'))  # 将中文字转化为URL链接。注意搜狗将中文字进行的GBK编码。而不是UTF-8
        urls = 'https://pinyin.sogou.com/dict/search/search_list/%s/normal/' % url_word  # 搜索链接
        response = requests.get(url=urls, headers={'User-Agent': self.ua.random}).text
        max_page = re.search('共有(.+)个搜索结果', response).group(1)
        m = re.findall('//pinyin.sogou.com/d/dict/download_cell.php?.+"', response)
        for page in range(2, math.ceil(int(max_page) / 10) + 1):
            response = requests.get(url=urls + str(page), headers={'User-Agent': self.ua.random}).text
            m.extend(re.findall('//pinyin.sogou.com/d/dict/download_cell.php?.+"', response))  # 将匹配到的下载链接装到链表中
        return map(lambda x: 'https:' + x[:-1], m)

    @staticmethod
    def _url_to_chinese(url):
        url_word = unquote(url)
        match = re.search('name=(.+)&|name=(.+)', url_word)
        name = match.group(1) if match.group(1) else match.group(2)
        return name, url_word

    def download_to_text(self, url_word):
        name, url_word = self._url_to_chinese(url_word)
        response = requests.get(url=url_word, headers={'User-Agent': self.ua.random})
        return name, self._to_txt(response.content)

    def download_to_scel(self, url_word, save_dir):
        if not os.path.isdir(save_dir):
            os.makedirs(save_dir)
        name, url_word = self._url_to_chinese(url_word)
        path = os.path.abspath(save_dir) + os.sep + name + '.scel'
        response = requests.get(url=url_word, headers={'User-Agent': self.ua.random})
        with open(path, mode='wb')as f:
            f.write(response.content)
        return path

    @staticmethod
    def _to_txt(data):
        ls_word = []  # 转化搜狗为UTF-8格式内容
        w = ''  # 每一个词条
        for i in range(0, len(data), 2):
            x = data[i:i + 2]  # 搜狗的UTF-8编码是两个字节
            t = struct.unpack('H', x)[0]  # 将其转化为无符号的短整形
            if 19968 < t < 40959 or t == 10:  # 判断是否是中文字符。10表示的是换行
                if t != 10:  # 不换行放在单个词条
                    w += chr(t)
                elif t == 10 and len(w):  # 换行且不等于空
                    ls_word.append(w)
                    w = ''
        return ls_word[1:]  # 第一行是注释。不需要，去除第一行

    def download_to_txt(self, url, txt_dir):
        if not os.path.isdir(txt_dir):
            os.makedirs(txt_dir)
        name, text = self.download_to_text(url)
        path = os.path.abspath(txt_dir) + os.sep + name + '.txt'
        with open(path, 'w', encoding='utf-8')as f:
            f.writelines(t + '\n' for t in text)
        return path


sg = SoGou()


def download_category(category_id, save_dir):
    category_url = 'https://pinyin.sogou.com/dict/cate/index/' + str(category_id)
    urls = sg.one_classify_lexicon(category_url)
    _dir = os.path.join(save_dir, str(category_id))
    if not os.path.exists(_dir):
        os.makedirs(_dir)

    for url in urls:
        try:
            logging.info('Download from %s', url)
            name, data = sg.download_to_text(url)
            name = name.replace(' ', '').replace('/', '_').strip().lower() + '.txt'
            f = os.path.join(_dir, name)
            logging.info('Save into %s', f)
            with open(f, mode='wt', encoding='utf8') as fout:
                for d in sorted(set(data)):
                    fout.write(d + '\n')
        except Exception as e:
            print(e)
            continue


def download_all_category(save_dir):
    for i in range(653):  # sogou has 653 categories in total
        download_category(i, save_dir)


def collect(save_dir, output_file, maxlen=6):
    files = []

    for d in os.listdir(save_dir):
        d = os.path.join(save_dir, d)
        if os.path.isdir(d):
            for f in os.listdir(d):
                files.append(os.path.join(d, f))

    files = sorted(set(files))
    print('Collect from {} files in total'.format(len(files)))
    vocabs = set()

    for f in files:
        if not os.path.exists(f):
            continue
        with open(f, mode='rt', encoding='utf8') as fin:
            for line in fin:
                line = line.strip('\n').strip()
                if not line:
                    continue
                if len(line) > maxlen:
                    continue
                vocabs.add(line)

    output_file = os.path.join(save_dir, output_file)
    with open(output_file, mode='wt', encoding='utf8') as fout:
        for v in sorted(vocabs):
            fout.write(v + '\n')



import re


def char_half2full(uchar):
    """Convert characters from half-width to full-width."""
    inside_code = ord(uchar)
    if inside_code < 0x0020 or inside_code > 0x7e:  # 不是半角字符就返回原来的字符
        return uchar
    if inside_code == 0x0020:  # 除了空格其他的全角半角的公式为:半角=全角-0xfee0
        inside_code = 0x3000
    else:
        inside_code += 0xfee0
    return chr(inside_code)


def string_half2full(s):
    return "".join([char_half2full(c) for c in s])


def b2q(s):
    """Convert character(s) from half-width to full-width."""
    if not s:
        return ""
    if len(s) == 1:
        return char_half2full(s)
    return string_half2full(s)


def char_full2half(uchar):
    """Convert characters from full-width to half-width."""
    inside_code = ord(uchar)
    if inside_code == 0x3000:
        inside_code = 0x0020
    else:
        inside_code -= 0xfee0
    if inside_code < 0x0020 or inside_code > 0x7e:  # 转完之后不是半角字符返回原来的字符
        return uchar
    return chr(inside_code)


def string_full2half(ustring):
    """Convert string from full-width to half-width."""
    return "".join([char_full2half(uchar) for uchar in ustring])


def q2b(s):
    """Convert character(s) from full-width to half-width."""
    if not s:
        return ""
    if len(s) == 1:
        return char_full2half(s)
    return string_full2half(s)


def split_sentence(text, reg=None):
    """Split long text to a list of sentence."""
    p = reg if reg is not None else '[\\n\\t\\s；。：;]+'
    return re.split(p, text)



import logging
import os

import jieba

from .abstract_tokenizer import AbstractTokenizer, VocabBasedTokenizer

ACCURATE_MODE = 0
FULL_MODE = 1
SEARCH_MODE = 2


class JiebaTokenizer(VocabBasedTokenizer):

    def __init__(self,
                 vocab_file,
                 userdict_files=None,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 bos_token='[BOS]',
                 eos_token='[EOS]',
                 **kwargs):
        super().__init__(
            vocab_file,
            pad_token=pad_token,
            unk_token=unk_token,
            bos_token=bos_token,
            eos_token=eos_token,
            **kwargs)
        jieba.initialize()
        if userdict_files:
            for f in userdict_files:
                if not os.path.exists(f):
                    logging.warning('Load userdict: {} failed. File does not exist. Skipped.'.format(f))
                    continue
                with open(f, mode='rt', encoding='utf8') as fin:
                    jieba.load_userdict(fin)
                    logging.info('Load userdict: {} finished.'.format(f))

    def tokenize(self, inputs, mode=ACCURATE_MODE, hmm=True, **kwargs):
        if mode == ACCURATE_MODE:
            return [t for t in jieba.cut(inputs, cut_all=False, HMM=hmm)]
        elif mode == FULL_MODE:
            return [t for t in jieba.cut(inputs, cut_all=True, HMM=hmm)]
        elif mode == SEARCH_MODE:
            return [t for t in jieba.cut_for_search(inputs, HMM=hmm)]
        else:
            raise ValueError('Invalid mode: {}'.format(mode))



import unittest

import jieba

from .jieba_tokenizer import ACCURATE_MODE, FULL_MODE, SEARCH_MODE, JiebaTokenizer


class JiebaTokenizerTest(unittest.TestCase):

    def testTokenize(self):
        tokenizer = JiebaTokenizer(
            vocab_file='testdata/vocab_chinese.txt',
            userdict_files=[
                'data/jieba/hello.txt',
                'naivenlp/tokenizers/data/dict.txt',
            ])

        sentences = [
            '我在上海工作',
            '我来到北京清华大学',
            '乒乓球拍卖完了',
            '中国科学技术大学',
        ]

        for sent in sentences:
            self.assertEqual(
                [t for t in jieba.cut(sent, cut_all=False, HMM=True)],
                tokenizer.tokenize(sent, mode=ACCURATE_MODE, hmm=True))
            self.assertEqual(
                [t for t in jieba.cut(sent, cut_all=False, HMM=False)],
                tokenizer.tokenize(sent, mode=ACCURATE_MODE, hmm=False))
            self.assertEqual(
                [t for t in jieba.cut(sent, cut_all=True, HMM=True)],
                tokenizer.tokenize(sent, mode=FULL_MODE, hmm=True))
            self.assertEqual(
                [t for t in jieba.cut(sent, cut_all=True, HMM=False)],
                tokenizer.tokenize(sent, mode=FULL_MODE, hmm=True))
            self.assertEqual(
                [t for t in jieba.cut_for_search(sent, HMM=True)],
                tokenizer.tokenize(sent, mode=SEARCH_MODE, hmm=True))
            self.assertEqual(
                [t for t in jieba.cut_for_search(sent, HMM=False)],
                tokenizer.tokenize(sent, mode=SEARCH_MODE, hmm=False))


if __name__ == "__main__":
    unittest.main()



import abc


class Node:

    def __init__(self):
        self.val = None
        self.children = {}


class AbstractTrie(abc.ABC):
    """Trie interface."""

    def put(self, sequence, **kwargs):
        """Insert an sequence"""
        raise NotImplementedError()

    def get(self, key, **kwargs):
        """Get node by key sequence"""
        raise NotImplementedError()

    def delete(self, key, **kwargs):
        """Delete an sequence"""
        raise NotImplementedError()

    def contains(self, key, **kwargs):
        """Contains an sequence or not"""
        raise NotImplementedError()

    def is_empty(self, **kwargs):
        """The trie is empty or not"""
        raise NotImplementedError()

    def longest_prefix_of(self, sequence, **kwargs):
        """Longest prefix of the sequence in the trie"""
        raise NotImplementedError()

    def keys_with_prefix(self, prefix, **kwargs):
        """Collect all of the sequence in the trie that has this prefix"""
        raise NotImplementedError()

    def size(self, **kwargs):
        """Number of leaf nodes of this trie"""
        raise NotImplementedError()

    def show(self, **kwargs):
        """Print this trie"""
        raise NotImplementedError()


class Trie(AbstractTrie):

    def __init__(self):
        self.root = Node()
        self.root.val = '.'

    def _put(self, node, sequence, depth):
        if depth == len(sequence):
            return
        v = sequence[depth]
        if v not in node.children:
            n = Node()
            n.val = v
            node.children[v] = n
        n = node.children[v]
        return self._put(n, sequence, depth + 1)

    def put(self, sequence, **kwargs):
        self._put(self.root, sequence, 0)

    def _get(self, node, sequence, depth):
        if depth == len(sequence):
            # return sequence
            return node
        if not sequence[depth] in node.children:
            return None
        n = node.children[sequence[depth]]
        return self._get(n, sequence, depth + 1)

    def get(self, sequence, **kwargs):
        return self._get(self.root, sequence, 0)

    def _delete(self, node, sequence, depth):
        if depth == len(sequence) - 1:
            node.children.pop(sequence[depth], None)
            return
        self._delete(node.children[sequence[depth]], sequence, depth + 1)

    def delete(self, sequence, **kwargs):
        return self._delete(self.root, sequence, 0)

    def contains(self, key, **kwargs):
        node = self.get(key)
        return True if node is not None else False

    def is_empty(self, **kwargs):
        return len(self.root.children) == 0

    def _search(self, node, sequence, depth):
        if not node.children:
            return sequence[:depth]
        if depth == len(sequence):
            return sequence

        if sequence[depth] not in node.children:
            return sequence[:depth]
        return self._search(node.children[sequence[depth]], sequence, depth + 1)

    def longest_prefix_of(self, sequence, **kwargs):
        return self._search(self.root, sequence, 0)

    def _collect(self, node, prefix):
        prefix = list(prefix)
        results = []

        def _traversal(n, arr):
            if not n.children:
                results.append(prefix + arr)
                return
            for k, c in n.children.items():
                arr.append(k)
                _traversal(c, arr[:])
                arr.pop()

        _traversal(node, [])
        return results

    def keys_with_prefix(self, prefix, **kwargs):
        node = self.get(prefix)
        if node is None:
            return []
        return self._collect(node, prefix)

    def size(self, **kwargs):

        def _count(node):
            if not node.children:
                return 1
            c = 0
            for k, n in node.children.items():
                c += _count(n)
            return c

        return _count(self.root)

    def _show(self, node, depth, max_depth=None):
        if not node:
            return
        if max_depth is not None and depth == max_depth + 1:
            return
        if depth == 0:
            print('.')
        for k, n in node.children.items():
            print("|    " * depth + '+----' + n.val)
            self._show(n, depth + 1, max_depth)

    def show(self, max_depth=None):
        self._show(self.root, 0, max_depth=max_depth)



import abc
import logging
import os


class AbstractTokenizer(abc.ABC):

    def tokenize(self, inputs, **kwargs):
        raise NotImplementedError()

    def tokens2ids(self, inputs, **kwargs):
        raise NotImplementedError()

    def ids2tokens(self, inputs, **kwargs):
        raise NotImplementedError()

    def encode(self, inputs, **kwargs):
        raise NotImplementedError()

    def decode(self, inputs, **kwargs):
        raise NotImplementedError()


class VocabBasedTokenizer(AbstractTokenizer):

    def __init__(self, vocab_file, **kwargs):
        self.vocab_file = vocab_file

        self._special_tokens = []
        for k, v in kwargs.items():
            if str(k).endswith('_token') and v is not None:
                self._special_tokens.append((k, v))

        self.vocab = self._build_vocab(vocab_file)
        self.reverse_vocab = self._reverse_vocab()

        for k, v in self._special_tokens:
            _id = str(k.split('_')[0]) + '_id'
            setattr(self, k, v)
            setattr(self, _id, self.vocab[v])

    def _build_vocab(self, file):
        if not file:
            logging.warning('vocab_file is empty or None.')
            return {}
        words = []
        with open(file, mode='rt', encoding='utf8') as fin:
            for line in fin:
                line = line.strip('\n').strip()
                if not line:
                    continue
                word = line.strip()
                words.append(word)

        vocab = set(words)
        special_tokens = [v for _, v in self._special_tokens]
        for t in special_tokens:
            if t not in vocab:
                words.append(t)

        d = {}
        for i in range(len(words)):
            d[words[i]] = i
        return d

    def _reverse_vocab(self):
        reverse_vocabs = {}
        for k, v in self.vocab.items():
            reverse_vocabs[v] = k
        return reverse_vocabs

    @property
    def vocab_size(self):
        return len(self.vocab)

    def special_tokens(self):
        return self._special_tokens

    def tokenize(self, inputs, **kwargs):
        raise NotImplementedError()

    def tokens2ids(self, tokens, add_bos=False, add_eos=False, **kwargs):
        ids = [self.vocab.get(t, self.unk_id) for t in tokens]
        if add_bos:
            ids = [self.bos_id] + ids
        if add_eos:
            ids = ids + [self.eos_id]
        return ids

    def ids2tokens(self, ids, drop_bos=False, drop_eos=False, **kwargs):
        tokens = [self.reverse_vocab.get(t, self.unk_token) for t in ids]
        if drop_bos and tokens[0] == self.bos_token:
            tokens = tokens[1:]
        if drop_eos and tokens[-1] == self.eos_token:
            tokens = tokens[:-1]
        return tokens

    def encode(self, inputs, add_bos=False, add_eos=False, **kwargs):
        tokens = self.tokenize(inputs, **kwargs)
        return self.tokens2ids(tokens, add_bos=add_bos, add_eos=add_eos, **kwargs)

    def decode(self, inputs, drop_bos=True, drop_eos=True, **kwargs):
        ids = [self.reverse_vocab.get(t, self.unk_token) for t in inputs]
        if not ids:
            return []
        return self.ids2tokens(ids, drop_bos=drop_bos, drop_eos=drop_eos, **kwargs)


class CustomTokenizer(VocabBasedTokenizer):

    def __init__(self,
                 vocab_file,
                 tokenize_fn,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 bos_token='[BOS]',
                 eos_token='[EOS]',
                 **kwargs):
        super().__init__(
            vocab_file,
            pad_token=pad_token,
            unk_token=unk_token,
            bos_token=bos_token,
            eos_token=eos_token,
            **kwargs)

        self.tokenize_fn = tokenize_fn

    def tokenize(self, inputs, **kwargs):
        return self.tokenize_fn(inputs, **kwargs)



import unittest

from .kenlm_corrector import KenLMCorrector


class KenLMCorrectorTest(unittest.TestCase):

    def testKenLMCorrector(self):
        c = KenLMCorrector()

        print('chinese char set size: {}'.format(len(c.chinese_char_set)))
        print('same pinyin map size: {}'.format(len(c.same_pinyin_map)))
        print('same stroke map size: {}'.format(len(c.same_stroke_map)))

        texts = [
            '软件开发工成师',
            '少先队员因该为老人让坐',
        ]

        for text in texts:
            print(c.correct(text))
            self.assertEqual(c.correct(text)[0], c.corrector.correct(text)[0])


if __name__ == "__main__":
    unittest.main()



import unittest

from .transformer_tokenizer import TransformerTokenizer


class TransformerTokenizerTest(unittest.TestCase):

    def testTransformerTokenizer(self):
        tokenizer = TransformerTokenizer(
            vocab_file='testdata/vocab_chinese.txt',
            bos_token='<S>',
            eos_token='<T>',)
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(100, tokenizer.unk_id)
        self.assertEqual(104, tokenizer.bos_id)
        self.assertEqual(105, tokenizer.eos_id)

        tokenizer = TransformerTokenizer(vocab_file='testdata/vocab_chinese.txt', bos_token='S', eos_token='</S>')
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(100, tokenizer.unk_id)
        self.assertEqual(21127, tokenizer.bos_id)
        self.assertEqual(21128, tokenizer.eos_id)


if __name__ == "__main__":
    unittest.main()



from .abstract_tokenizer import AbstractTokenizer, CustomTokenizer
from .bert_tokenizer import BertTokenizer
from .jieba_tokenizer import JiebaTokenizer
from .transformer_tokenizer import TransformerTokenizer



# Lint as python3
# Copyright 2018 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
# pylint: disable=g-import-not-at-top
"""Utilities for file download and caching."""
from __future__ import absolute_import, division, print_function

import errno
import functools
import gc
import hashlib
import multiprocessing
import multiprocessing.dummy
import os
import random
import shutil
import signal
import sys
import tarfile
import threading
import time
import weakref
import zipfile
from abc import abstractmethod
from contextlib import closing

import numpy as np
import six
from six.moves.urllib.error import HTTPError, URLError
from six.moves.urllib.request import urlopen

try:
    import typing

    def is_iterator(x):
        return isinstance(x, typing.Iterator)
except ImportError:
    # Python2 uses next, and Python3 should have typing so __next__ is not needed.
    def is_iterator(x):
        return hasattr(x, '__iter__') and hasattr(x, 'next')


if sys.version_info[0] == 2:

    def urlretrieve(url, filename, reporthook=None, data=None):
        """Replacement for `urlretrieve` for Python 2.
        Under Python 2, `urlretrieve` relies on `FancyURLopener` from legacy
        `urllib` module, known to have issues with proxy management.
        Arguments:
            url: url to retrieve.
            filename: where to store the retrieved data locally.
            reporthook: a hook function that will be called once on establishment of
              the network connection and once after each block read thereafter. The
              hook will be passed three arguments; a count of blocks transferred so
              far, a block size in bytes, and the total size of the file.
            data: `data` argument passed to `urlopen`.
        """

        def chunk_read(response, chunk_size=8192, reporthook=None):
            content_type = response.info().get('Content-Length')
            total_size = -1
            if content_type is not None:
                total_size = int(content_type.strip())
            count = 0
            while True:
                chunk = response.read(chunk_size)
                count += 1
                if reporthook is not None:
                    reporthook(count, chunk_size, total_size)
                if chunk:
                    yield chunk
                else:
                    break

        response = urlopen(url, data)
        with open(filename, 'wb') as fd:
            for chunk in chunk_read(response, reporthook=reporthook):
                fd.write(chunk)
else:
    from six.moves.urllib.request import urlretrieve


def _extract_archive(file_path, path='.', archive_format='auto'):
    """Extracts an archive if it matches tar, tar.gz, tar.bz, or zip formats.
    Arguments:
        file_path: path to the archive file
        path: path to extract the archive file
        archive_format: Archive format to try for extracting the file.
            Options are 'auto', 'tar', 'zip', and None.
            'tar' includes tar, tar.gz, and tar.bz files.
            The default 'auto' is ['tar', 'zip'].
            None or an empty list will return no matches found.
    Returns:
        True if a match was found and an archive extraction was completed,
        False otherwise.
    """
    if archive_format is None:
        return False
    if archive_format == 'auto':
        archive_format = ['tar', 'zip']
    if isinstance(archive_format, six.string_types):
        archive_format = [archive_format]

    for archive_type in archive_format:
        if archive_type == 'tar':
            open_fn = tarfile.open
            is_match_fn = tarfile.is_tarfile
        if archive_type == 'zip':
            open_fn = zipfile.ZipFile
            is_match_fn = zipfile.is_zipfile

        if is_match_fn(file_path):
            with open_fn(file_path) as archive:
                try:
                    archive.extractall(path)
                except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
                    if os.path.exists(path):
                        if os.path.isfile(path):
                            os.remove(path)
                        else:
                            shutil.rmtree(path)
                    raise
            return True
    return False


def get_file(fname,
             origin,
             untar=False,
             md5_hash=None,
             file_hash=None,
             cache_subdir='datasets',
             hash_algorithm='auto',
             extract=False,
             archive_format='auto',
             cache_dir=None):
    """Downloads a file from a URL if it not already in the cache.
    By default the file at the url `origin` is downloaded to the
    cache_dir `~/.keras`, placed in the cache_subdir `datasets`,
    and given the filename `fname`. The final location of a file
    `example.txt` would therefore be `~/.keras/datasets/example.txt`.
    Files in tar, tar.gz, tar.bz, and zip formats can also be extracted.
    Passing a hash will verify the file after download. The command line
    programs `shasum` and `sha256sum` can compute the hash.
    Arguments:
        fname: Name of the file. If an absolute path `/path/to/file.txt` is
            specified the file will be saved at that location.
        origin: Original URL of the file.
        untar: Deprecated in favor of 'extract'.
            boolean, whether the file should be decompressed
        md5_hash: Deprecated in favor of 'file_hash'.
            md5 hash of the file for verification
        file_hash: The expected hash string of the file after download.
            The sha256 and md5 hash algorithms are both supported.
        cache_subdir: Subdirectory under the Keras cache dir where the file is
            saved. If an absolute path `/path/to/folder` is
            specified the file will be saved at that location.
        hash_algorithm: Select the hash algorithm to verify the file.
            options are 'md5', 'sha256', and 'auto'.
            The default 'auto' detects the hash algorithm in use.
        extract: True tries extracting the file as an Archive, like tar or zip.
        archive_format: Archive format to try for extracting the file.
            Options are 'auto', 'tar', 'zip', and None.
            'tar' includes tar, tar.gz, and tar.bz files.
            The default 'auto' is ['tar', 'zip'].
            None or an empty list will return no matches found.
        cache_dir: Location to store cached files, when None it
            defaults to the [Keras
              Directory](/faq/#where-is-the-keras-configuration-filed-stored).
    Returns:
        Path to the downloaded file
    """
    if cache_dir is None:
        cache_dir = os.path.join(os.path.expanduser('~'), '.keras')
    if md5_hash is not None and file_hash is None:
        file_hash = md5_hash
        hash_algorithm = 'md5'
    datadir_base = os.path.expanduser(cache_dir)
    if not os.access(datadir_base, os.W_OK):
        datadir_base = os.path.join('/tmp', '.keras')
    datadir = os.path.join(datadir_base, cache_subdir)
    _makedirs_exist_ok(datadir)

    if untar:
        untar_fpath = os.path.join(datadir, fname)
        fpath = untar_fpath + '.tar.gz'
    else:
        fpath = os.path.join(datadir, fname)

    download = False
    if os.path.exists(fpath):
        # File found; verify integrity if a hash was provided.
        if file_hash is not None:
            if not validate_file(fpath, file_hash, algorithm=hash_algorithm):
                print('A local file was found, but {} file hash not match original value of {}.'.format(
                    hash_algorithm, file_hash))
                download = True
    else:
        download = True

    if download:
        print('Downloading data from', origin)

        class ProgressTracker(object):
            # Maintain progbar for the lifetime of download.
            # This design was chosen for Python 2.7 compatibility.
            progbar = None

        def dl_progress(count, block_size, total_size):
            if ProgressTracker.progbar is None:
                if total_size == -1:
                    total_size = None
                ProgressTracker.progbar = Progbar(total_size)
            else:
                ProgressTracker.progbar.update(count * block_size)

        error_msg = 'URL fetch failure on {}: {} -- {}'
        try:
            try:
                urlretrieve(origin, fpath, dl_progress)
            except HTTPError as e:
                raise Exception(error_msg.format(origin, e.code, e.msg))
            except URLError as e:
                raise Exception(error_msg.format(origin, e.errno, e.reason))
        except (Exception, KeyboardInterrupt) as e:
            if os.path.exists(fpath):
                os.remove(fpath)
            raise e
        ProgressTracker.progbar = None

    if untar:
        if not os.path.exists(untar_fpath):
            _extract_archive(fpath, datadir, archive_format='tar')
        return untar_fpath

    if extract:
        _extract_archive(fpath, datadir, archive_format)

    return fpath


def _makedirs_exist_ok(datadir):
    if six.PY2:
        # Python 2 doesn't have the exist_ok arg, so we try-except here.
        try:
            os.makedirs(datadir)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise
    else:
        os.makedirs(datadir, exist_ok=True)  # pylint: disable=unexpected-keyword-arg


def _hash_file(fpath, algorithm='sha256', chunk_size=65535):
    """Calculates a file sha256 or md5 hash.
    Example:
    ```python
    _hash_file('/path/to/file.zip')
    'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
    ```
    Arguments:
        fpath: path to the file being validated
        algorithm: hash algorithm, one of 'auto', 'sha256', or 'md5'.
            The default 'auto' detects the hash algorithm in use.
        chunk_size: Bytes to read at a time, important for large files.
    Returns:
        The file hash
    """
    if (algorithm == 'sha256') or (algorithm == 'auto' and len(hash) == 64):
        hasher = hashlib.sha256()
    else:
        hasher = hashlib.md5()

    with open(fpath, 'rb') as fpath_file:
        for chunk in iter(lambda: fpath_file.read(chunk_size), b''):
            hasher.update(chunk)

    return hasher.hexdigest()


def validate_file(fpath, file_hash, algorithm='auto', chunk_size=65535):
    """Validates a file against a sha256 or md5 hash.
    Arguments:
        fpath: path to the file being validated
        file_hash:  The expected hash string of the file.
            The sha256 and md5 hash algorithms are both supported.
        algorithm: Hash algorithm, one of 'auto', 'sha256', or 'md5'.
            The default 'auto' detects the hash algorithm in use.
        chunk_size: Bytes to read at a time, important for large files.
    Returns:
        Whether the file is valid
    """
    if (algorithm == 'sha256') or (algorithm == 'auto' and len(file_hash) == 64):
        hasher = 'sha256'
    else:
        hasher = 'md5'

    if str(_hash_file(fpath, hasher, chunk_size)) == str(file_hash):
        return True
    else:
        return False


class Progbar(object):
    """Displays a progress bar.
    Arguments:
        target: Total number of steps expected, None if unknown.
        width: Progress bar width on screen.
        verbose: Verbosity mode, 0 (silent), 1 (verbose), 2 (semi-verbose)
        stateful_metrics: Iterable of string names of metrics that should *not* be
          averaged over time. Metrics in this list will be displayed as-is. All
          others will be averaged by the progbar before display.
        interval: Minimum visual progress update interval (in seconds).
        unit_name: Display name for step counts (usually "step" or "sample").
    """

    def __init__(self,
                 target,
                 width=30,
                 verbose=1,
                 interval=0.05,
                 stateful_metrics=None,
                 unit_name='step'):
        self.target = target
        self.width = width
        self.verbose = verbose
        self.interval = interval
        self.unit_name = unit_name
        if stateful_metrics:
            self.stateful_metrics = set(stateful_metrics)
        else:
            self.stateful_metrics = set()

        self._dynamic_display = ((hasattr(sys.stdout, 'isatty') and
                                  'ipykernel' in sys.modules or
                                  'posix' in sys.modules or
                                  'PYCHARM_HOSTED' in os.environ))
        self._total_width = 0
        self._seen_so_far = 0
        # We use a dict + list to avoid garbage collection
        # issues found in OrderedDict
        self._values = {}
        self._values_order = []
        self._start = time.time()
        self._last_update = 0

    def update(self, current, values=None, finalize=None):
        """Updates the progress bar.
        Arguments:
            current: Index of current step.
            values: List of tuples: `(name, value_for_last_step)`. If `name` is in
              `stateful_metrics`, `value_for_last_step` will be displayed as-is.
              Else, an average of the metric over time will be displayed.
            finalize: Whether this is the last update for the progress bar. If
              `None`, defaults to `current >= self.target`.
        """
        if finalize is None:
            if self.target is None:
                finalize = False
            else:
                finalize = current >= self.target

        values = values or []
        for k, v in values:
            if k not in self._values_order:
                self._values_order.append(k)
            if k not in self.stateful_metrics:
                # In the case that progress bar doesn't have a target value in the first
                # epoch, both on_batch_end and on_epoch_end will be called, which will
                # cause 'current' and 'self._seen_so_far' to have the same value. Force
                # the minimal value to 1 here, otherwise stateful_metric will be 0s.
                value_base = max(current - self._seen_so_far, 1)
                if k not in self._values:
                    self._values[k] = [v * value_base, value_base]
                else:
                    self._values[k][0] += v * value_base
                    self._values[k][1] += value_base
            else:
                # Stateful metrics output a numeric value. This representation
                # means "take an average from a single value" but keeps the
                # numeric formatting.
                self._values[k] = [v, 1]
        self._seen_so_far = current

        now = time.time()
        info = ' - %.0fs' % (now - self._start)
        if self.verbose == 1:
            if now - self._last_update < self.interval and not finalize:
                return

            prev_total_width = self._total_width
            if self._dynamic_display:
                sys.stdout.write('\b' * prev_total_width)
                sys.stdout.write('\r')
            else:
                sys.stdout.write('\n')

            if self.target is not None:
                numdigits = int(np.log10(self.target)) + 1
                bar = ('%' + str(numdigits) + 'd/%d [') % (current, self.target)
                prog = float(current) / self.target
                prog_width = int(self.width * prog)
                if prog_width > 0:
                    bar += ('=' * (prog_width - 1))
                    if current < self.target:
                        bar += '>'
                    else:
                        bar += '='
                bar += ('.' * (self.width - prog_width))
                bar += ']'
            else:
                bar = '%7d/Unknown' % current

            self._total_width = len(bar)
            sys.stdout.write(bar)

            if current:
                time_per_unit = (now - self._start) / current
            else:
                time_per_unit = 0

            if self.target is None or finalize:
                if time_per_unit >= 1 or time_per_unit == 0:
                    info += ' %.0fs/%s' % (time_per_unit, self.unit_name)
                elif time_per_unit >= 1e-3:
                    info += ' %.0fms/%s' % (time_per_unit * 1e3, self.unit_name)
                else:
                    info += ' %.0fus/%s' % (time_per_unit * 1e6, self.unit_name)
            else:
                eta = time_per_unit * (self.target - current)
                if eta > 3600:
                    eta_format = '%d:%02d:%02d' % (eta // 3600,
                                                   (eta % 3600) // 60, eta % 60)
                elif eta > 60:
                    eta_format = '%d:%02d' % (eta // 60, eta % 60)
                else:
                    eta_format = '%ds' % eta

                info = ' - ETA: %s' % eta_format

            for k in self._values_order:
                info += ' - %s:' % k
                if isinstance(self._values[k], list):
                    avg = np.mean(self._values[k][0] / max(1, self._values[k][1]))
                    if abs(avg) > 1e-3:
                        info += ' %.4f' % avg
                    else:
                        info += ' %.4e' % avg
                else:
                    info += ' %s' % self._values[k]

            self._total_width += len(info)
            if prev_total_width > self._total_width:
                info += (' ' * (prev_total_width - self._total_width))

            if finalize:
                info += '\n'

            sys.stdout.write(info)
            sys.stdout.flush()

        elif self.verbose == 2:
            if finalize:
                numdigits = int(np.log10(self.target)) + 1
                count = ('%' + str(numdigits) + 'd/%d') % (current, self.target)
                info = count + info
                for k in self._values_order:
                    info += ' - %s:' % k
                    avg = np.mean(self._values[k][0] / max(1, self._values[k][1]))
                    if avg > 1e-3:
                        info += ' %.4f' % avg
                    else:
                        info += ' %.4e' % avg
                info += '\n'

                sys.stdout.write(info)
                sys.stdout.flush()

        self._last_update = now

    def add(self, n, values=None):
        self.update(self._seen_so_far + n, values)



import os

import pycorrector

from .abstract_corrector import AbstractCorrector


class KenLMCorrector(AbstractCorrector):

    def __init__(self,
                 kenlm_model_path=None,
                 word_freq_files=None,
                 confusion_files=None,
                 same_pinyin_files=None,
                 same_stroke_files=None,
                 stop_word_files=None,
                 ):
        super().__init__()
        if kenlm_model_path:
            self.corrector = pycorrector.Corrector(language_model_path=kenlm_model_path)
        else:
            self.corrector = pycorrector.Corrector()
        self.corrector.check_corrector_initialized()
        self.chinese_char_set = self.corrector.cn_char_set
        self.same_pinyin_map = self.corrector.same_pinyin
        self.same_stroke_map = self.corrector.same_stroke
        self.confusion_map = self.corrector.custom_confusion
        self.word_freq_map = self.corrector.word_freq
        self.stop_words_map = self.corrector.stopwords

        if word_freq_files:
            for f in word_freq_files:
                if not os.path.exists(f):
                    continue
                self.corrector.set_custom_word(f)

        if confusion_files:
            for f in confusion_files:
                if not os.path.exists(f):
                    continue
                self.corrector.set_custom_confusion_dict(f)

        if same_pinyin_files:
            for f in same_pinyin_files:
                if not os.path.exists(f):
                    continue
                m = self.corrector.load_same_pinyin(f, sep='\t')
                self.corrector.same_pinyin.update(m)

        if same_stroke_files:
            for f in same_stroke_files:
                if not os.path.exists(f):
                    continue
                m = self.corrector.load_same_stroke(f, sep='\t')
                self.corrector.same_stroke.update(m)

        if stop_word_files:
            for f in stop_word_files:
                if not os.path.exists(f):
                    continue
                d = self.corrector.load_word_freq_dict(f)
                self.stop_words_map.update(d)

    def correct(self, text, **kwargs):
        corrected, details = self.corrector.correct(text)
        res = []
        for d in details:
            res.append((d[0], d[1], d[2], d[3]))
        return corrected, res



import unittest

from .trie import Trie


class TrieTest(unittest.TestCase):

    def testTrie(self):
        t = Trie()
        t.put("上海市 浦东新区".split())
        t.put("上海市浦东新区")
        print()
        t.show()

        self.assertEqual('市', t.get('上海市').val)
        self.assertEqual('浦东新区', t.get('上海市 浦东新区'.split()).val)
        self.assertEqual('市', t.get('上海市').val)
        self.assertEqual('区', t.get('上海市浦东新区').val)
        self.assertEqual(None, t.get('上海市浦东区'))
        self.assertEqual(None, t.get('上海市浦东新区哈哈哈'))

        self.assertEqual(2, t.size())

        self.assertEqual(True, t.contains('上海市 浦东新区'.split()))
        self.assertEqual(True, t.contains('上海市浦东新区'))
        self.assertEqual(True, t.contains('上海市浦东'))

        self.assertEqual('上海市浦东新区', t.longest_prefix_of('上海市浦东新区'))
        self.assertEqual('上海市浦东新', t.longest_prefix_of('上海市浦东新'))
        self.assertEqual(['上海市', '浦东新区'], t.longest_prefix_of('上海市 浦东新区'.split()))

        t.put('上海市黄浦区')
        for r in t.keys_with_prefix('上海市'):
            print(r)
            print('=' * 80)
        self.assertEqual(3, t.size())

        t.delete('上海市浦东')
        t.show()

        self.assertEqual(False, t.is_empty())

        print('=' * 80)

        t.delete('上')
        t.show()

        t.delete('上海市'.split())
        t.show()
        self.assertEqual(True, t.is_empty())


if __name__ == "__main__":
    unittest.main()



from strsimpy import (
    Cosine,
    Damerau,
    Jaccard,
    Levenshtein,
    LongestCommonSubsequence,
    NormalizedLevenshtein,
    OptimalStringAlignment,
    SorensenDice,
    WeightedLevenshtein,
)
from strsimpy.weighted_levenshtein import default_deletion_cost, default_insertion_cost, default_substitution_cost

DEFAULT_K = 3
DEFAULT_COSINE = Cosine(k=DEFAULT_K)
DEFAULT_JACCARD = Jaccard(k=DEFAULT_K)
DEFAULT_LEVENSHTEIN = Levenshtein()
DEFAULT_NORMALIZED_LEVENSHTEIN = NormalizedLevenshtein()
DEFAULT_WEIGHTED_LEVENSHTEIN = WeightedLevenshtein()
DEFAULT_DAMERAU = Damerau()
DEFAULT_LCS = LongestCommonSubsequence()
DEFAULT_SORENSEN_DICE = SorensenDice(k=DEFAULT_K)
DEFAULT_OSA = OptimalStringAlignment()


def cosine_distance(a, b, k=DEFAULT_K):
    cosine = DEFAULT_COSINE if k == DEFAULT_K else Cosine(k=k)
    return cosine.distance(a, b)


def cosine_similarity(a, b, k=DEFAULT_K):
    cosine = DEFAULT_COSINE if k == DEFAULT_K else Cosine(k=k)
    return cosine.distance(a, b)


def jaccard_distance(a, b, k=DEFAULT_K):
    jaccard = DEFAULT_JACCARD if k == DEFAULT_K else Jaccard(k=k)
    return jaccard.distance(a, b)


def jaccard_similarity(a, b, k=DEFAULT_K):
    jaccard = DEFAULT_JACCARD if k == DEFAULT_K else Jaccard(k=k)
    return jaccard.similarity(a, b)


def levenshtein_distance(a, b):
    return DEFAULT_LEVENSHTEIN.distance(a, b)


def levenshtein_distance_normalized(a, b):
    return DEFAULT_NORMALIZED_LEVENSHTEIN.distance(a, b)


def levenshtein_similarity(a, b):
    return DEFAULT_NORMALIZED_LEVENSHTEIN.similarity(a, b)


def weighted_levenshtein_distance(a, b, substitution_cost_fn=None, insertion_cost_fn=None, deletion_cost_fn=None):
    substitution_fn = substitution_cost_fn or default_substitution_cost
    insertion_fn = insertion_cost_fn or default_insertion_cost
    deletion_fn = deletion_cost_fn or default_deletion_cost
    wl = WeightedLevenshtein(
        substitution_cost_fn=substitution_fn,
        insertion_cost_fn=insertion_fn,
        deletion_cost_fn=deletion_fn)
    return wl.distance(a, b)


def damerau_distance(a, b):
    return DEFAULT_DAMERAU.distance(a, b)


def lcs_distance(a, b):
    return DEFAULT_LCS.distance(a, b)


def lcs_length(a, b):
    return DEFAULT_LCS.length(a, b)


def longest_common_subsequence_distance(a, b):
    return DEFAULT_LCS.distance(a, b)


def longest_common_subsequence_length(a, b):
    return DEFAULT_LCS.length(a, b)


def sorense_dice_distance(a, b, k=DEFAULT_K):
    sorense_dice = DEFAULT_SORENSEN_DICE if k == DEFAULT_K else SorensenDice(k=k)
    return sorense_dice.distance(a, b)


def sorense_dice_similarity(a, b, k=DEFAULT_K):
    sorense_dice = DEFAULT_SORENSEN_DICE if k == DEFAULT_K else SorensenDice(k=k)
    return sorense_dice.similarity(a, b)


def osa_distance(a, b):
    return DEFAULT_OSA.distance(a, b)


def optimal_string_alignment_distance(a, b):
    return DEFAULT_OSA.distance(a, b)



import unittest


from .deep_corrector import TransformerCorrector


class DeepCorrectorTest(unittest.TestCase):

    def testTransformerCorrector(self):
        c = TransformerCorrector(saved_model='models/correction_models/transformer-step-2000')
        inputs = [
            '我最近每天晚上都会拧着鼻子去喝30cc的醋了。',
            '我拍了一张樱花的照片。',
            '它是星星形状的娇小的糖果，带着昔日的风貌。',
            '羡慕啊。',
            '你重视旋律还是重视歌词？',
            '今天我去看了电影。',
            '即使那样的一天，他们也不打伞，跑回家。',
            '我走啦！',
            '吃寿司卷的时候，不要一言不发。',
            '这个网点Lang8的朋友告诉我的。',
        ]

        for inp in inputs:
            print('inpiut: ', inp)
            res, prob = c.correct(inp)
            print('result: ', res)
            print('  prob: ', prob)
            print()


if __name__ == "__main__":
    unittest.main()



from .abstract_tokenizer import VocabBasedTokenizer
from .tokenizer import BasicTokenizer, WordpieceTokenizer


class TransformerTokenizer(VocabBasedTokenizer):

    def __init__(self,
                 vocab_file,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 bos_token='[BOS]',
                 eos_token='[EOS]',
                 do_lower_case=True,
                 do_basic_tokenization=True,
                 tokenize_chinese_chars=True,
                 never_split=None,
                 max_input_chars_per_word=100,
                 **kwargs):
        super().__init__(
            vocab_file, pad_token=pad_token, unk_token=unk_token, bos_token=bos_token, eos_token=eos_token, **kwargs)
        self.do_lower_case = do_lower_case
        self.do_basic_tokenization = do_basic_tokenization
        self.tokenize_chinese_chars = tokenize_chinese_chars
        self.never_split = never_split
        self.max_input_chars_per_word = max_input_chars_per_word

        if self.do_basic_tokenization:
            self.basic_tokenizer = BasicTokenizer(
                do_lower_case=do_lower_case,
                tokenize_chinese_chars=tokenize_chinese_chars,
                never_split=never_split,
            )
        else:
            self.basic_tokenizer = None

        self.wordpiece_tokenizer = WordpieceTokenizer(
            vocab=set(self.vocab.keys()),
            unk_token=self.unk_token,
            max_input_chars_per_word=self.max_input_chars_per_word)

    def tokenize(self, inputs, never_split=None, **kwargs):
        tokens = []
        never_split = never_split + self.never_split if never_split is not None else self.never_split
        if self.do_basic_tokenization:
            for token in self.basic_tokenizer.tokenize(inputs, never_split=never_split):
                for t in self.wordpiece_tokenizer.tokenize(token):
                    tokens.append(t)
        else:
            tokens = self.wordpiece_tokenizer.tokenize(inputs)

        return tokens



import unittest

from .bert_tokenizer import BertTokenizer


class BertTokenizerTest(unittest.TestCase):

    def testBertTokenizer(self):
        tokenizer = BertTokenizer(
            vocab_file='testdata/vocab_chinese.txt',
            bos_token='<S>',
            eos_token='<T>')

        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(100, tokenizer.unk_id)
        self.assertEqual(104, tokenizer.bos_id)
        self.assertEqual(105, tokenizer.eos_id)
        self.assertEqual(101, tokenizer.cls_id)
        self.assertEqual(102, tokenizer.sep_id)
        self.assertEqual(103, tokenizer.mask_id)

        tokenizer = BertTokenizer(vocab_file='testdata/vocab_chinese.txt', bos_token='<S>', eos_token='</S>')
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(100, tokenizer.unk_id)
        self.assertEqual(104, tokenizer.bos_id)
        self.assertEqual(21127, tokenizer.eos_id)
        self.assertEqual(101, tokenizer.cls_id)
        self.assertEqual(102, tokenizer.sep_id)
        self.assertEqual(103, tokenizer.mask_id)

        tokenizer = BertTokenizer(
            vocab_file='testdata/vocab_chinese.txt', bos_token='<S>', eos_token='</S>', xxx_token='XXX')
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(100, tokenizer.unk_id)
        self.assertEqual(104, tokenizer.bos_id)
        self.assertEqual(21127, tokenizer.eos_id)
        self.assertEqual(101, tokenizer.cls_id)
        self.assertEqual(102, tokenizer.sep_id)
        self.assertEqual(103, tokenizer.mask_id)
        self.assertEqual(21128, tokenizer.xxx_id)


if __name__ == "__main__":
    unittest.main()



import logging

from naivenlp.similarity import (
    cosine_distance,
    cosine_similarity,
    damerau_distance,
    jaccard_distance,
    jaccard_similarity,
    lcs_distance,
    lcs_length,
    levenshtein_distance,
    levenshtein_distance_normalized,
    levenshtein_similarity,
    longest_common_subsequence_distance,
    longest_common_subsequence_length,
    optimal_string_alignment_distance,
    osa_distance,
    sorense_dice_distance,
    sorense_dice_similarity,
    weighted_levenshtein_distance,
)
from naivenlp.structures.trie import AbstractTrie, Node, Trie
from naivenlp.tokenizers.abstract_tokenizer import AbstractTokenizer, CustomTokenizer, VocabBasedTokenizer
from naivenlp.tokenizers.bert_tokenizer import BertTokenizer
from naivenlp.tokenizers.tokenizer import BasicTokenizer, WordpieceTokenizer
from naivenlp.tokenizers.transformer_tokenizer import TransformerTokenizer
from naivenlp.utils.texts import b2q, q2b, split_sentence

__version__ = "0.0.9"
__name__ = "naivenlp"


logging.basicConfig(
    format="%(asctime)s %(levelname)7s %(filename)20s %(lineno)4d] %(message)s",
    level=logging.INFO
)



import os
import logging

import tensorflow as tf

from .abstract_corrector import AbstractCorrector


class DeepCorrector(AbstractCorrector):

    def __init__(self, saved_model, **kwargs):
        loaded = tf.saved_model.load(saved_model)
        logging.info('Load saved model from {} successfully.'.format(saved_model))
        self.model = loaded.signatures['serving_default']

    def correct(self, text, **kwargs):
        raise NotImplementedError()


class TransformerCorrector(DeepCorrector):

    def correct(self, text, **kwargs):
        inputs = tf.constant([' '.join(text[:])], dtype=tf.string)
        outputs = self.model(inputs)
        tokens, probs = outputs['text'].numpy()[0], outputs['log_probs'].numpy()[0]
        tokens = ''.join(''.join([t.decode('utf-8') for t in tokens]).split())
        return tokens, probs[0]



from .abstract_tokenizer import VocabBasedTokenizer
from .tokenizer import BasicTokenizer, WordpieceTokenizer
from .transformer_tokenizer import TransformerTokenizer


class BertTokenizer(TransformerTokenizer):

    def __init__(self,
                 vocab_file,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 bos_token='[BOS]',
                 eos_token='[EOS]',
                 cls_token='[CLS]',
                 sep_token='[SEP]',
                 mask_token='[MASK]',
                 do_lower_case=True,
                 do_basic_tokenization=True,
                 tokenize_chinese_chars=True,
                 never_split=None,
                 max_input_chars_per_word=100,
                 **kwargs):

        super().__init__(
            vocab_file,
            pad_token=pad_token,
            unk_token=unk_token,
            bos_token=bos_token,
            eos_token=eos_token,
            cls_token=cls_token,
            sep_token=sep_token,
            mask_token=mask_token,
            do_lower_case=True,
            do_basic_tokenization=True,
            tokenize_chinese_chars=True,
            never_split=None,
            max_input_chars_per_word=100,
            **kwargs)
